// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file ResidualParallelEstimator.h */

/** \defgroup Estimator Estimator module
 * @{ <img src="estimator.png"> @}
 */

#ifndef AMDIS_RESIDUALPARALLELESTIMATOR_H
#define AMDIS_RESIDUALPARALLELESTIMATOR_H

#ifdef _OPENMP

#include <vector>
#include "ResidualEstimator.h"

namespace AMDiS {

  /**
   * \ingroup Estimator
   * 
   * \brief
   * Parallel Residual Estimator for scalar problems.
   */
  class ResidualParallelEstimator : public Estimator
  {
  public:
    /// Creator class used in the OEMSolverMap.
    class Creator : public EstimatorCreator
    {
    public:
      Creator() : EstimatorCreator() {}

      virtual ~Creator() {}

      /// Returns a new ODirSolver object.
      Estimator* create() 
      { 
	return new ResidualParallelEstimator(name, row);
      }
    };

    /// Constructor.
    ResidualParallelEstimator(std::string name, int r);

    /// Destructor.
    ~ResidualParallelEstimator();

    ///
    virtual void addSystem(DOFMatrix *matrix_,
			   DOFVector<double> *uh_,
			   DOFVector<double> *fh_,
			   DOFVector<double> *uhOld_ = NULL);

    ///
    virtual void addUhOldToSystem(int system, DOFVector<double> *uhOld_);

    ///
    virtual double estimate(double timestep);

  private:
    std::vector<ResidualEstimator*> seqEstimators_;

    /// Constant in fromt of the time
    double C3;
  };
  
}

#endif  // _OPENMP

#endif // AMDIS_RESIDUALPARALLELESTIMATOR_H
