//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include "VertexVector.h"
#include "DOFAdmin.h"
#include "DOFIterator.h"

namespace AMDiS {

  VertexVector::VertexVector(const DOFAdmin *a, std::string n)
    : DOFVectorDOF()
  {
    name = n;
    feSpace = NULL;
    admin = a;
    const_cast<DOFAdmin*>(admin)->addDOFIndexed(this);
    const_cast<DOFAdmin*>(admin)->addDOFContainer(this);
  }

  VertexVector::~VertexVector()
  {
    const_cast<DOFAdmin*>(admin)->removeDOFIndexed(this);
    const_cast<DOFAdmin*>(admin)->removeDOFContainer(this);
  }

  void VertexVector::set(DegreeOfFreedom val)
  {
    DOFIteratorBase it(const_cast<DOFAdmin*>(admin), USED_DOFS);
    for (it.reset(); !it.end(); ++it)
      if (!it.isDOFFree()) 
	operator[](it.getDOFIndex()) = val;
  } 

  void VertexVector::print()
  {
    DOFIteratorBase it(const_cast<DOFAdmin*>(admin), USED_DOFS);
    for (it.reset(); !it.end(); ++it)
      if (!it.isDOFFree())
	std::cout << "DOF " << it.getDOFIndex() << " -> " 
		  << operator[](it.getDOFIndex()) << "\n";
  }

  void VertexVector::changeDofIndices(std::map<int, int>& dofIndexMap)
  {
    FUNCNAME("VertexVector::changedofIndices()");

    std::vector<DegreeOfFreedom> tmp(vec.size());
    for (int i = 0; i < static_cast<int>(tmp.size()); i++)
      tmp[i] = i;
    
    for (std::map<int, int>::iterator it = dofIndexMap.begin(); 
	 it != dofIndexMap.end(); ++it)
      if (vec[it->first] == -1)
	tmp[it->second] = -1;
      else 
	tmp[it->second] = dofIndexMap[vec[it->first]];

    vec = tmp;
  }

}
