//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include "parallel/ParallelProblemStatBase.h"
#include "parallel/MeshDistributor.h"
#include "parallel/MpiHelper.h"

namespace AMDiS {

  void ParallelProblemStatBase::buildAfterCoarsen(AdaptInfo *adaptInfo, Flag flag,
						  bool assembleMatrix,
						  bool assembleVector)
  {
    FUNCNAME("ParallelProblemStatBase::buildAfterCoarsen()");

    meshDistributor->checkMeshChange();
    ProblemVec::buildAfterCoarsen(adaptInfo, flag, assembleMatrix, assembleVector);

    double vm, rss;
    processMemUsage(vm, rss);   
    vm /= 1024.0;
    rss /= 1024.0;
    
    MSG("My memory usage is VM = %f MB    RSS = %f MB\n", vm, rss);
    
    mpi::globalAdd(vm);
    mpi::globalAdd(rss);

    MSG("Overall memory usage is VM = %f MB    RSS = %f MB\n", vm, rss);
  }
  
  
  void ParallelProblemStatBase::addToMeshDistributor(MeshDistributor& m)
  {
    meshDistributor = &m;
    m.addProblemStat(this);
  }


  void ParallelProblemStatBase::processMemUsage(double& vm_usage, 
						double& resident_set)
  {
    using std::ios_base;
    using std::ifstream;
    using std::string;
    
    vm_usage     = 0.0;
    resident_set = 0.0;
    
    // 'file' stat seems to give the most reliable results    
    ifstream stat_stream("/proc/self/stat",ios_base::in);
    
    // dummy vars for leading entries in stat that we don't care about
    string pid, comm, state, ppid, pgrp, session, tty_nr;
    string tpgid, flags, minflt, cminflt, majflt, cmajflt;
    string utime, stime, cutime, cstime, priority, nice;
    string O, itrealvalue, starttime;
    
    // the two fields we want    
    unsigned long vsize;
    long rss;
    
    stat_stream >> pid >> comm >> state >> ppid >> pgrp >> session >> tty_nr
		>> tpgid >> flags >> minflt >> cminflt >> majflt >> cmajflt
		>> utime >> stime >> cutime >> cstime >> priority >> nice
		>> O >> itrealvalue >> starttime >> vsize >> rss;

    // in case x86-64 is configured to use 2MB pages    
    long page_size_kb = sysconf(_SC_PAGE_SIZE) / 1024;
    vm_usage     = vsize / 1024.0;
    resident_set = rss * page_size_kb;
  }

  
}
