//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include "StdMpi.h"

namespace AMDiS {

  int intSizeOf(int &data)
  {
    return 1;
  }

  int intSizeOf(std::map<const DegreeOfFreedom*, DegreeOfFreedom> &data)
  {
    return data.size() * 2;
  }

  int intSizeOf(std::map<WorldVector<double>, int> &data)
  {
    return data.size() * (Global::getGeo(WORLD) + 1);
  }

  int intSizeOf(std::vector<MeshStructure> &data)
  {
    int s = 0;
    for (unsigned int i = 0; i < data.size(); i++)
      s += data[i].getCode().size() + 2;

    return s;
  }

  int intSizeOf(std::vector<WorldVector<double> > &data)
  {
    return data.size() * Global::getGeo(WORLD);
  }

  int intSizeOf(std::vector<int> &data)
  {
    return data.size();
  }

  int intSizeOf(std::vector<double> &data)
  {
    return data.size();
  }
  
  int intSizeOf(std::vector<std::pair<int, int> > &data)
  {
    return data.size() * 2;
  }

  int intSizeOf(std::vector<AtomicBoundary> &data)
  {
    return data.size() * 6;
  }

  int intSizeOf(std::vector<BoundaryObject> &data)
  {
    return data.size() * 5;
  }

  int intSizeOf(std::vector<const DegreeOfFreedom*> &data)
  {
    return data.size();
  }

  int intSizeOf(std::vector<std::vector<double> > &data)
  {
    int size = 1;

    for (unsigned int i = 0; i < data.size(); i++)
      size += data[i].size() + 1;
    
    return size;
  }

  int intSizeOf(std::map<BoundaryType, std::map<DegreeOfFreedom, DegreeOfFreedom> > &data)
  {
    int size = 1;

    for (std::map<BoundaryType, std::map<DegreeOfFreedom, DegreeOfFreedom> >::iterator it = data.begin();
	 it != data.end(); ++it) {
      size += 2 + it->second.size() * 2;
    }

    MSG("RET SIZE = %d\n", size);

    return size;
  }

  void makeBuf(int &data, int *buf)
  {
    buf[0] = data;
  }

  void makeFromBuf(int &data, int *buf, int bufSize)
  {
    data = buf[0];
  }

  void makeBuf(std::map<const DegreeOfFreedom*, DegreeOfFreedom> &data, int* buf)
  {
    int i = 0;
    for (std::map<const DegreeOfFreedom*, DegreeOfFreedom>::iterator it = data.begin();
	 it != data.end(); ++it) {
      buf[i++] = *(it->first);
      buf[i++] = it->second;
    }
  }
  
  void makeFromBuf(std::vector<std::pair<DegreeOfFreedom, DegreeOfFreedom> > &data, 
		   int *buf, int bufSize)
  {    
    if (bufSize == 0)
      return;

    TEST_EXIT(bufSize % 2 == 0)("This should not happen!\n");

    data.clear();
    data.reserve(bufSize / 2);

    for (int i = 0; i < (bufSize / 2); i++)
      data.push_back(std::make_pair(buf[i * 2], buf[i * 2 + 1]));
  }

  void makeBuf(std::map<WorldVector<double>, int> &data, double* buf)
  {
    int i = 0;
    for (std::map<WorldVector<double>, int>::iterator it = data.begin();
	 it != data.end(); ++it) {
      for (int j = 0; j < Global::getGeo(WORLD); j++)
	buf[i++] = it->first[j];
      buf[i++] = static_cast<double>(it->second);      
    }
  }

  void makeFromBuf(std::map<WorldVector<double>, int> &data, double* buf, int bufSize)
  {
    if (bufSize == 0)
      return;

    int oneEntrySize = Global::getGeo(WORLD) + 1;
    int nEntry = bufSize / oneEntrySize;

    TEST_EXIT(bufSize % oneEntrySize == 0)("This should not happen!\n");

    data.clear();
    int i = 0;
    WorldVector<double> coords;

    for (int j = 0; j < nEntry; j++) {
      for (int k = 0; k < Global::getGeo(WORLD); k++)
	coords[k] = buf[i++];
      data[coords] = static_cast<int>(buf[i++]);
    }
  }

  void makeBuf(std::vector<MeshStructure> &data, uint64_t *buf)
  {    
    int pos = 0;
    for (unsigned int i = 0; i < data.size(); i++) {
      buf[pos++] = data[i].getCode().size();
      buf[pos++] = data[i].getNumElements();

      for (unsigned int j = 0; j < data[i].getCode().size(); j++)
	buf[pos++] = data[i].getCode()[j];
    }
  }

  void makeFromBuf(std::vector<MeshStructure> &data, 
		   uint64_t *buf, int bufSize)
  {
    int pos = 0;

    while (pos < bufSize) {       
      int codeSize = buf[pos++];
      int nElements = buf[pos++];
      std::vector<uint64_t> code;
      code.resize(codeSize);
      for (int i = 0; i < codeSize; i++)
	code[i] = buf[pos++];

      MeshStructure meshCode;
      meshCode.init(code, nElements);

      data.push_back(meshCode);
    }	
  }

  void makeBuf(std::vector<WorldVector<double> > &data, double *buf)
  {
    int dimOfWorld = Global::getGeo(WORLD);
    int pos = 0;
    for (unsigned int i = 0; i < data.size(); i++)
      for (int j = 0; j < dimOfWorld; j++)
	buf[pos++] = data[i][j];
  }

  void makeFromBuf(std::vector<WorldVector<double> > &data, double *buf, int bufSize)
  {
    int dimOfWorld = Global::getGeo(WORLD);
    TEST_EXIT(bufSize % Global::getGeo(WORLD) == 0)("This should not happen!\n");

    int pos = 0;
    data.resize(bufSize / Global::getGeo(WORLD));
    for (unsigned int i = 0; i < data.size(); i++)
      for (int j = 0; j < dimOfWorld; j++)
	data[i][j] = buf[pos++];
  }

  void makeBuf(std::vector<int> &data, int *buf)
  {
    for (unsigned int i = 0; i < data.size(); i++)
      buf[i] = data[i];
  }

  void makeFromBuf(std::vector<int> &data, int *buf, int bufSize)
  {
    data.resize(bufSize);

    for (int i = 0; i < bufSize; i++)
      data[i] = buf[i];
  }

  void makeBuf(std::vector<double> &data, double *buf)
  {
    for (unsigned int i = 0; i < data.size(); i++)
      buf[i] = data[i];
  }

  void makeFromBuf(std::vector<double> &data, double *buf, int bufSize)
  {
    data.resize(bufSize);

    for (int i = 0; i < bufSize; i++)
      data[i] = buf[i];
  }

  void makeBuf(std::vector<std::pair<int, int> > &data, int *buf)
  {
    for (unsigned int i = 0; i < data.size(); i++) {
      buf[i * 2] = data[i].first;
      buf[i * 2 + 1] = data[i].second;
    }
  }

  void makeBuf(std::vector<AtomicBoundary> &data, int *buf)
  {
    for (unsigned int i = 0; i < data.size(); i++) {
      buf[i * 6] = data[i].rankObj.elIndex;
      buf[i * 6 + 1] = data[i].rankObj.subObj;
      buf[i * 6 + 2] = data[i].rankObj.ithObj;
      buf[i * 6 + 3] = data[i].neighObj.elIndex;
      buf[i * 6 + 4] = data[i].neighObj.subObj;
      buf[i * 6 + 5] = data[i].neighObj.ithObj;
    }
  }

  void makeFromBuf(std::vector<AtomicBoundary> &data, int *buf, int bufSize)
  {
    if (bufSize == 0)
      return;

    TEST_EXIT(bufSize % 6 == 0)("This should not happen!\n");    

    data.resize(bufSize / 6);
    for (int i = 0; i < bufSize / 6; i++) {
      data[i].rankObj.elIndex = buf[i * 6];
      data[i].rankObj.subObj = static_cast<GeoIndex>(buf[i * 6 + 1]);
      data[i].rankObj.ithObj = buf[i * 6 + 2];
      data[i].neighObj.elIndex = buf[i * 6 + 3];
      data[i].neighObj.subObj = static_cast<GeoIndex>(buf[i * 6 + 4]);
      data[i].neighObj.ithObj = buf[i * 6 + 5];
    }
  }

  void makeBuf(std::vector<BoundaryObject> &data, int *buf)
  {
    for (unsigned int i = 0; i < data.size(); i++) {
      buf[i * 5] = data[i].elIndex;
      buf[i * 5 + 1] = data[i].elType;
      buf[i * 5 + 2] = data[i].subObj;
      buf[i * 5 + 3] = data[i].ithObj;
      buf[i * 5 + 4] = data[i].reverseMode;
    }
  }

  void makeFromBuf(std::vector<BoundaryObject> &data, int *buf, int bufSize)
  {
    if (bufSize == 0)
      return;

    TEST_EXIT(bufSize % 5 == 0)("This should not happen!\n");    

    data.resize(bufSize / 5);
    for (int i = 0; i < bufSize / 5; i++) {
      data[i].elIndex = buf[i * 5];
      data[i].elType = buf[i * 5 + 1];
      data[i].subObj = static_cast<GeoIndex>(buf[i * 5 + 2]);
      data[i].ithObj = buf[i * 5 + 3];
      data[i].reverseMode = static_cast<bool>(buf[i * 5 + 4]);
    }
  }

  void makeBuf(std::vector<std::vector<double> > &data, double *buf)
  {
    buf[0] = data.size();
    int counter = 1;

    for (unsigned int i = 0; i < data.size(); i++) {
      buf[counter++] = data[i].size();
      for (unsigned int j = 0; j < data[i].size(); j++)
	buf[counter++] = data[i][j];
    }
  }

  void makeFromBuf(std::vector<std::vector<double> > &data, double *buf, int bufSize)
  {
    data.resize(buf[0]);
    int counter = 1;

    for (unsigned int i = 0; i < data.size(); i++) {
      data[i].resize(buf[counter++]);
      
      for (unsigned int j = 0; j < data[i].size(); j++)
	data[i][j] = buf[counter++];
    }
  }

  void makeBuf(std::map<BoundaryType, std::map<DegreeOfFreedom, DegreeOfFreedom> > &data, int *buf)
  {
    buf[0] = data.size();
    int counter = 1;

    for (std::map<BoundaryType, std::map<DegreeOfFreedom, DegreeOfFreedom> >::iterator it = data.begin();
	 it != data.end(); ++it) {
      buf[counter++] = it->first;
      buf[counter++] = it->second.size();

      for (std::map<DegreeOfFreedom, DegreeOfFreedom>::iterator it2 = it->second.begin();
	   it2 != it->second.end(); ++it2) {
	buf[counter++] = it2->first;
	buf[counter++] = it2->second;
      }
    }
  }

  void makeFromBuf(std::map<BoundaryType, std::map<DegreeOfFreedom, DegreeOfFreedom> > &data, int *buf, int bufSize)
  {
    data.clear();

    if (bufSize == 0)
      return;

    int counter = 1;
    
    for (int i = 0; i < buf[0]; i++) {
      BoundaryType bound = buf[counter++];
      std::map<DegreeOfFreedom, DegreeOfFreedom> dofs;

      int nDofs = buf[counter++];
      for (int j = 0; j < nDofs; j++) {
	DegreeOfFreedom dof0, dof1;
	dof0 = buf[counter++];
	dof1 = buf[counter++];
	dofs[dof0] = dof1;
      }

      data[bound] = dofs;
    }

    TEST_EXIT(bufSize == counter)("Should not happen!\n");
  }
}
